function [] = run(dataName,objective,method,outFile)
% dataName = {'manMade','denoise'}
% objective = {'PL','MF','LBP'}
% method = {'BFGS','SGD','ASGD','SMD'}
% outFile = name of matFile to save paths of nll, testerr, and weights to

fprintf('Loading Data\n');
switch dataName
    case 'manMade'
        trainNdx = 1:108;
        testNdx = 1:129;
        batch_size = 3;
        [traindata,testdata,featureEng] = loadmanMade(trainNdx,testNdx);
    case 'denoise'
        trainNdx = 1:40;
        testNdx = 1:10;
        batch_size = 1;
        [traindata,testdata,featureEng] = loadDenoise(trainNdx,testNdx+max(trainNdx));
end

% Parameters:
reg = 1;
nstates = 2;
maxIter = 40;

switch objective
    case 'PL'
        eta0 = 0.04
        gradArgs = {featureEng, traindata, reg};
        gradFunc = @pseudoLikGradient;
        gradFuncWeights = @pseudoLikGradientWeights;
    case 'MF';
        if strcmp(dataName,'manMade')
            eta0 = 0.0001;
        elseif strcmp(dataName, 'denoise')
            eta0 = 0.001;
        end
        infEng = latticeInferMF(nstates);
        gradFunc = @scrfGradient;
        gradArgs = {featureEng, infEng, traindata, reg};
        gradFuncWeights = @scrfGradientWeights;
    case 'LBP'
        eta0 = 0.0001;
        infEng = latticeInferBP(nstates);
        gradFunc = @scrfGradient;
        gradArgs = {featureEng, infEng, traindata, reg};
        gradFuncWeights = @scrfGradientWeights;
end
displayFn = @displayIter;
displayArgs = {};

% SMD parameters
lambda = 0.9;
mu = 10*eta0;

fprintf('Initializing Weights\n');
winit = initialWeights(traindata,reg,featureEng);

fprintf('Training CRF (and Computing NLL path)\n');
switch method
    case 'SGD'
        [weights, nll, errTrace, weightsTrace] = ...
            stochgrad(gradFunc,winit,trainNdx,'gradArgs',gradArgs,...
            'maxIter',maxIter,'eta',eta0,'batch_size',batch_size,...
            'displayFn',displayFn,'displayArgs',displayArgs);
    case 'ASGD'
        [weights, nll, errTrace, weightsTrace] = ...
            stochgrad(gradFunc,winit,trainNdx,'gradArgs',gradArgs,...
            'maxIter',maxIter,'eta',eta0,'batch_size',batch_size,...
            'displayFn',displayFn,'displayArgs',displayArgs,'anneal',1);
    case 'SMD'
        [weights, nll, errTrace, weightsTrace] = ...
            smd(gradFunc,winit,trainNdx,'gradArgs',gradArgs,...
            'maxIter',maxIter,'eta0',eta0,'lambda',lambda,'mu',mu,...
            'batch_size',batch_size,'displayFn',displayFn,...
            'displayArgs',displayArgs);
    case 'BFGS'
        options = optimset('Display','iter','Diagnostics','off',...
            'GradObj','on','LargeScale','off','MaxFunEval',maxIter);
        % @scrfGradientWeights stores the weights and err at each iteration
        % in global variables
        global FMINweightsTrace FMINerrTrace
        FMINweightsTrace = [];FMINweightsTrace(:,1) = winit;
        FMINerrTrace = [];FMINerrTrace(1) = feval(gradFunc,winit, trainNdx, gradArgs{:});
        [weights, nll] = fminunc(gradFuncWeights, winit, options, trainNdx, gradArgs{:});
        if length(FMINerrTrace) < maxIter+1
            for i = length(FMINerrTrace)+1:maxIter+1
                FMINweightsTrace(:,i) = FMINweightsTrace(:,i-1);
                FMINerrTrace(i) = FMINerrTrace(i-1);
            end
        end
        errTrace=FMINerrTrace;weightsTrace=FMINweightsTrace;
end

fprintf('Calculating Test Error Path\n');

infEng = latticeInferBP(nstates);
for i = 1:maxIter+1
    testErrTrace(i) = classifPerformance(weightsTrace(:,i),testNdx,featureEng,infEng,testdata);
    fprintf('Iter = %d, testErr = %.3f\n',i-1,testErrTrace(i));
end

save(outFile,'dataName','objective','method',...
    'errTrace','testErrTrace','weightsTrace');